import * as ec2 from '@aws-cdk/aws-ec2';
import * as iam from '@aws-cdk/aws-iam';
import * as cdk from '@aws-cdk/core';
/**
 * Construct properties for the EKS-D cluster.
 *
 * @stability stable
 */
export interface ClusterProps {
    /**
     * VPC for the cluster.
     *
     * @default - get or create a VPC
     * @stability stable
     */
    readonly vpc?: ec2.IVpc;
    /**
     * The default EC2 instance type.
     *
     * @default t3.large
     * @stability stable
     */
    readonly defaultInstanceType?: ec2.InstanceType;
    /**
     * number of instances.
     *
     * @default 1
     * @stability stable
     */
    readonly capacitySize?: number;
    /**
     * AMI for the EKS-D instance node.
     *
     * @default - The latest AMI from ubuntu-focal-20.04-amd64-server
     * @stability stable
     */
    readonly machineImage?: ec2.IMachineImage;
    /**
     * Print AMI ID in the output.
     *
     * @default - true
     * @stability stable
     */
    readonly outputAmiId?: boolean;
    /**
     * Create EC2 spot instnce for the cluster node.
     *
     * @default - false
     * @stability stable
     */
    readonly spot?: boolean;
}
/**
 * @stability stable
 */
export declare enum SpotInstanceType {
    /**
     * @stability stable
     */
    ONE_TIME = "one-time",
    /**
     * @stability stable
     */
    PERSISTENT = "persistent"
}
/**
 * @stability stable
 */
export declare enum InstanceInterruptionBehavior {
    /**
     * @stability stable
     */
    HIBERNATE = "hibernate",
    /**
     * @stability stable
     */
    STOP = "stop",
    /**
     * @stability stable
     */
    TERMINATE = "terminate"
}
/**
 * @stability stable
 */
export declare enum BlockDurationMinutes {
    /**
     * @stability stable
     */
    ONE_HOUR = 60,
    /**
     * @stability stable
     */
    TWO_HOURS = 120,
    /**
     * @stability stable
     */
    THREE_HOURS = 180,
    /**
     * @stability stable
     */
    FOUR_HOURS = 240,
    /**
     * @stability stable
     */
    FIVE_HOURS = 300,
    /**
     * @stability stable
     */
    SIX_HOURS = 360
}
/**
 * @stability stable
 */
export interface SpotOptions {
    /**
     * @stability stable
     */
    readonly spotInstanceType?: SpotInstanceType;
    /**
     * @stability stable
     */
    readonly validUntil?: string;
    /**
     * @stability stable
     */
    readonly maxPrice?: string;
    /**
     * @stability stable
     */
    readonly instanceInterruptionBehavior?: InstanceInterruptionBehavior;
    /**
     * @stability stable
     */
    readonly blockDurationMinutes?: BlockDurationMinutes;
}
/**
 * @stability stable
 */
export interface LaunchTemplateOptions {
    /**
     * @stability stable
     */
    readonly machineImage: ec2.IMachineImage;
    /**
     * @stability stable
     */
    readonly instanceType: ec2.InstanceType;
    /**
     * @stability stable
     */
    readonly spotOptions?: SpotOptions;
    /**
     * @stability stable
     */
    readonly userData: ec2.UserData;
    /**
     * @stability stable
     */
    readonly instanceProfile?: iam.CfnInstanceProfile;
}
/**
 * @stability stable
 */
export interface AutoScalingGroupOptions {
    /**
     * The vpc for the AutoScalingGroup.
     *
     * @stability stable
     */
    readonly vpc: ec2.IVpc;
    /**
     * Launch template options for the AutoScalingGroup.
     *
     * @stability stable
     */
    readonly launchTemplate: LaunchTemplateOptions;
    /**
     * default capacity size for the Auto Scaling Group.
     *
     * @default 1
     * @stability stable
     */
    readonly defaultCapacitySize?: number;
}
/**
 * Represents the EKS-D cluster.
 *
 * @stability stable
 */
export declare class Cluster extends cdk.Construct {
    private readonly defaultInstanceType;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props?: ClusterProps);
    private _createAutoScalingGroup;
}
/**
 * The AMI provider to get the latest Ubuntu Linux AMI.
 *
 * @stability stable
 */
export declare class UbumtuAmiProvider {
    readonly scope: cdk.Construct;
    readonly userData: ec2.UserData;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, userData: ec2.UserData);
    /**
     * @stability stable
     */
    get amiId(): ec2.IMachineImage;
}
